import requests
from .errors import ApiError
from .helpers import base_url
from .data import quote

def orders(account_id, access_token):
    headers = {'Authorization': f"Bearer {access_token}"}
    # params = {'accountId': account_id}
    params = {}

    # r = requests.get(base_url(f'/orders'), headers=headers, params=params)
    r = requests.get(base_url(f'/accounts/{account_id}/orders'), headers=headers, params=params)
    if r.status_code != 200:
        # raise ApiError(f"GET /orders", r.status_code, r.json()['error'])
        raise ApiError(f"GET /accounts/{account_id}/orders", r.status_code, r.json()['error'])

    return r.json()

def place_conditional_order(account_id, symbol, price_to_sell, quantity, access_token, buy_price=None):
    price = 0

    if not buy_price:
        stock_quote = quote(symbol, access_token)
        price = stock_quote[symbol]['askPrice']
    else:
        price = buy_price


    headers = {'Authorization': f"Bearer {access_token}"}

    data = {
        "orderType": "LIMIT",
        "session": "NORMAL",
        "price": price,
        "duration": "DAY",
        "orderStrategyType": "TRIGGER",
        "orderLegCollection": [
            {
                "instruction": "BUY",
                "quantity": quantity,
                "instrument": {
                    "symbol": symbol,
                    "assetType": "EQUITY"
                }
            }
        ],
        "childOrderStrategies": [
            {
                "orderType": "LIMIT",
                "session": "NORMAL",
                "price": price_to_sell,
                "duration": "DAY",
                "orderStrategyType": "SINGLE",
                    "orderLegCollection": [
                        {
                            "instruction": "SELL",
                            "quantity": quantity,
                            "instrument": {
                                "symbol": symbol,
                                "assetType": "EQUITY"
                            }
                        }
                    ]
            }
        ]
    }

    r = requests.post(base_url(f'/accounts/{account_id}/orders'), headers=headers, json=data)
    if r.status_code != 200:
        raise ApiError(f"POST /accounts/{account_id}/orders", r.status_code, r.json()['error'])

    return r.json()

def buy_limit(account_id, symbol, price, quantity, access_token):

    headers = {'Authorization': f"Bearer {access_token}"}

    data = {
        "orderType": "LIMIT",
        "session": "NORMAL",
        "price": price,
        "duration": "DAY",
        "orderStrategyType": "SINGLE",
        "orderLegCollection": [
            {
                "instruction": "BUY",
                "quantity": quantity,
                "instrument": {
                    "symbol": symbol,
                    "assetType": "EQUITY"
                }
            }
        ]
    }

    r = requests.post(base_url(f'/accounts/{account_id}/orders'), headers=headers, json=data)
    if r.status_code != 200:
        raise ApiError(f"POST /accounts/{account_id}/orders", r.status_code, r.json()['error'])

    return r.json()

def place_conditional_order_data(account_id, symbol, price_to_sell, quantity, access_token, buy_price=None):
    price = 0

    if not buy_price:
        stock_quote = quote(symbol, access_token)
        price = stock_quote[symbol]['askPrice']
    else:
        price = buy_price

    data = {
        "orderType": "LIMIT",
        "session": "NORMAL",
        "price": price,
        "duration": "DAY",
        "orderStrategyType": "TRIGGER",
        "orderLegCollection": [
            {
                "instruction": "BUY",
                "quantity": quantity,
                "instrument": {
                    "symbol": symbol,
                    "assetType": "EQUITY"
                }
            }
        ],
        "childOrderStrategies": [
            {
                "orderType": "LIMIT",
                "session": "NORMAL",
                "price": price_to_sell,
                "duration": "DAY",
                "orderStrategyType": "SINGLE",
                    "orderLegCollection": [
                        {
                            "instruction": "SELL",
                            "quantity": quantity,
                            "instrument": {
                                "symbol": symbol,
                                "assetType": "EQUITY"
                            }
                        }
                    ]
            }
        ]
    }

    return data



from .auth import access_token
from .errors import ApiError
from .data import quote, search, price_history, fundamental
from .accounts import account, transactions, positions
from .helpers import api_call
from .trading import orders, place_conditional_order_data, place_conditional_order, buy_limit
from .watchlist import watchlists, watchlist

class client:

    def __init__(self, refresh_token, consumer_key, account_id=""):
        self.refresh_token = refresh_token
        self.consumer_key = consumer_key
        self.account_id = account_id

        self.refresh_access_token()

    def refresh_access_token(self):
        try:
            self.access_token = access_token(self.refresh_token, self.consumer_key)
        except ApiError as error:
            raise ApiError(f"Error authenticating: {error.args[2]}")

    @api_call
    def quote(self, ticker):
        return quote(ticker, self.access_token)

    @api_call
    def search(self, symbol):
        return search(symbol, self.access_token)

    @api_call
    def account(self):
        return account(self.account_id, self.access_token)

    @api_call
    def orders(self):
        return orders(self.account_id, self.access_token)

    @api_call
    def transactions(self):
        return transactions(self.account_id, self.access_token)

    @api_call
    def watchlists(self):
        return watchlists(self.account_id, self.access_token)

    @api_call
    def watchlist(self, watchlist_id):
        return watchlist(watchlist_id, self.account_id, self.access_token)

    @api_call
    def fundamental(self, symbol):
        return fundamental(symbol, self.access_token)

    @api_call
    def price_history(self, symbol, *, period_type='day', period=1, frequency_type='minute', frequency=1, start_date=None, end_date=None, need_extended_hours_data=False):
        return price_history(symbol, self.access_token, period_type=period_type, period=period, frequency_type=frequency_type, frequency=frequency, start_date=start_date, end_date=end_date, need_extended_hours_data=need_extended_hours_data)

    @api_call
    def place_conditional_order_data(self, symbol, price_to_sell, quantity, buy_price=None):
        return place_conditional_order_data(self.account_id, symbol, price_to_sell, quantity, self.access_token, buy_price=buy_price)

    @api_call
    def positions(self):
        return positions(self.account_id, self.access_token)

    @api_call
    def place_conditional_order(self, symbol, price_to_sell, quantity, buy_price=None):
        return place_conditional_order(self.account_id, symbol, price_to_sell, quantity, self.access_token, buy_price=buy_price)

    @api_call
    def buy_limit(self, symbol, price, quantity):
        return buy_limit(self.account_id, symbol, price, quantity, self.access_token)



from .errors import ApiError

def base_url(path):
    return 'https://api.tdameritrade.com/v1' + path

# Wraps the api call so that authentication is handled correctly
def api_call(func):
    def wrapped(*args, **kwargs):
        self = args[0]
        try:
            return func(*args, **kwargs)
        except ApiError as error:
            self.refresh_access_token()
            return func(*args, **kwargs)

    return wrapped



import requests
from .errors import ApiError
from .helpers import base_url

"""Returns an access token when given a refresh token and consumer key"""
def access_token(refresh_token, consumer_key):
    data = {'grant_type': 'refresh_token', 'refresh_token': refresh_token, 'client_id': consumer_key}

    r = requests.post(base_url('/oauth2/token'), data=data)
    if r.status_code != 200:
        raise ApiError(f"POST /oauth2/token/", r.status_code, r.json()['error'])
    return r.json()['access_token']




import requests
from .errors import ApiError
from .helpers import base_url

def quote(ticker, access_token):
    headers = {'Authorization': f"Bearer {access_token}"}

    r = requests.get(base_url(f'/marketdata/{ticker}/quotes'), headers=headers)
    if r.status_code != 200:
        raise ApiError(f"GET /marketdata/{ticker}/quotes", r.status_code, r.json()['error'])

    return r.json()

def search(symbol, access_token):
    headers = {'Authorization': f"Bearer {access_token}"}
    params = {'symbol': symbol, 'projection': 'symbol-search'}

    r = requests.get(base_url('/instruments'), headers=headers, params=params)
    if r.status_code != 200:
        raise ApiError(f"GET /instruments", r.status_code, r.json()['error'])

    return r.json()

def fundamental(symbol, access_token):
    headers = {'Authorization': f"Bearer {access_token}"}
    params = {'symbol': symbol, 'projection': 'fundamental'}

    r = requests.get(base_url('/instruments'), headers=headers, params=params)
    if r.status_code != 200:
        raise ApiError(f"GET /instruments", r.status_code, r.json()['error'])

    return r.json()

# Start and end date are time since epoch. Use periods if not using start and end date.
def price_history(symbol, access_token, *, period_type='day', period=1, frequency_type='minute', frequency=1, start_date=None, end_date=None, need_extended_hours_data=False):
    headers = {'Authorization': f"Bearer {access_token}"}
    params = { 'periodType': period_type, 'frequencyType': frequency_type, 'frequency': frequency, 'needExtendendHoursData': need_extended_hours_data }
    if start_date and end_date:
        params.update({'startDate': start_date, 'endDate': end_date})
    else:
        params.update({'period': period})

    r = requests.get(base_url(f'/marketdata/{symbol}/pricehistory'), headers=headers, params=params)
    if r.status_code != 200:
        raise ApiError(f"GET /marketdata/{symbol}/pricehistory", r.status_code, r.json()['error'])

    return r.json()



import requests
from .errors import ApiError
from .helpers import base_url

def account(account_id, access_token):
    headers = {'Authorization': f"Bearer {access_token}"}
    params = {'fields': 'positions,orders'}

    r = requests.get(base_url(f'/accounts/{account_id}'), headers=headers, params=params)
    if r.status_code != 200:
        raise ApiError(f"GET /accounts/{account_id}", r.status_code, r.json()['error'])

    return r.json()

def transactions(account_id, access_token):
    headers = {'Authorization': f"Bearer {access_token}"}

    r = requests.get(base_url(f'/accounts/{account_id}/transactions'), headers=headers)
    if r.status_code != 200:
        raise ApiError(f"GET /accounts/{account_id}/transactions", r.status_code, r.json()['error'])

    return r.json()

def positions(account_id, access_token):
    headers = {'Authorization': f"Bearer {access_token}"}
    params = {'fields': 'positions,orders'}

    r = requests.get(base_url(f'/accounts/{account_id}'), headers=headers, params=params)
    if r.status_code != 200:
        raise ApiError(f"GET /accounts/{account_id}", r.status_code, r.json()['error'])

    return r.json()['securitiesAccount']['positions']



import requests
from .errors import ApiError
from .helpers import base_url

def watchlists(account_id, access_token):
    headers = {'Authorization': f"Bearer {access_token}"}

    r = requests.get(base_url(f'/accounts/{account_id}/watchlists'), headers=headers)
    if r.status_code != 200:
        raise ApiError(f"GET /accounts/{account_id}/watchlists", r.status_code, r.json()['error'])

    return r.json()

def watchlist(watchlist_id, account_id, access_token):
    headers = {'Authorization': f"Bearer {access_token}"}

    r = requests.get(base_url(f'/accounts/{account_id}/watchlists/{watchlist_id}'), headers=headers)
    if r.status_code != 200:
        raise ApiError(f"GET /accounts/{account_id}/watchlists/{watchlist_id}", r.status_code, r.json()['error'])

    return r.json()
